@@ -166,6 +166,23 @@ char *senna_enc_mysql(sen_encoding encoding)
   }
   return 0;
 }
+
+#ifdef USE_QUERY_ABORT
+static int senna_check_abort(void *)
+{
+  return current_thd->killed == THD::KILL_CONNECTION;
+}
+
+static void senna_set_abort_handler(MYISAM_SHARE *share)
+{
+  for (uint i = 0; i < (uint)share->state.header.keys; i++) {
+    if (share->keyinfo[i].senna != NULL) {
+      sen_index_set_abort_callback(share->keyinfo[i].senna, senna_check_abort,
+				   NULL);
+    }
+  }
+}
+#endif /* USE_QUERY_ABORT */
 #endif /* ENABLE_SENNA */
 
 /*
@@ -212,12 +229,12 @@ int table2myisam(TABLE *table_arg, MI_KEYDEF **keydef_out,
   pos= table_arg->key_info;
   for (i= 0; i < share->keys; i++, pos++)
   {
-    keydef[i].flag= (pos->flags & (HA_NOSAME | HA_FULLTEXT | HA_SPATIAL));
+    keydef[i].flag= ((uint16) pos->flags & (HA_NOSAME | HA_FULLTEXT | HA_SPATIAL));
 #ifdef ENABLE_SENNA
     if ((keydef[i].flag & HA_FULLTEXT)) {
       if (table_arg->key_info) {
 	keydef[i].senna_flags=pos->senna_flags;
-        keydef[i].senna_initial_n_segments=pos->senna_initial_n_segments;
+	keydef[i].senna_initial_n_segments=pos->senna_initial_n_segments;
 	keydef[i].senna_encoding=senna_enc_senna(pos->senna_encoding);
       } else {
 	keydef[i].senna_flags=SEN_DISABLE_SENNA;
@@ -695,6 +712,11 @@ int ha_myisam::open(const char *name, int mode, uint test_if_locked)
     int_table_flags|=HA_REC_NOT_IN_SEQ;
   if (file->s->options & (HA_OPTION_CHECKSUM | HA_OPTION_COMPRESS_RECORD))
     int_table_flags|=HA_HAS_CHECKSUM;
+#ifdef ENABLE_SENNA
+#ifdef USE_QUERY_ABORT
+  senna_set_abort_handler(file->s);
+#endif
+#endif
   return (0);
 }
 
@@ -1465,7 +1487,7 @@ void ha_myisam::start_bulk_insert(ha_rows rows)
   DBUG_ENTER("ha_myisam::start_bulk_insert");
   THD *thd= current_thd;
   ulong size= min(thd->variables.read_buff_size,
-                  table->s->avg_row_length*rows);
+                  (ulong) (table->s->avg_row_length*rows));
   DBUG_PRINT("info",("start_bulk_insert: rows %lu size %lu",
                      (ulong) rows, size));
 
@@ -1655,10 +1677,14 @@ int ha_myisam::index_next_same(byte * buf,
 			       const byte *key __attribute__((unused)),
 			       uint length __attribute__((unused)))
 {
+  int error;
   DBUG_ASSERT(inited==INDEX);
   statistic_increment(table->in_use->status_var.ha_read_next_count,
-		      &LOCK_status);
-  int error=mi_rnext_same(file,buf);
+                      &LOCK_status);
+  do
+  {
+    error= mi_rnext_same(file,buf);
+  } while (error == HA_ERR_RECORD_DELETED);
   table->status=error ? STATUS_NOT_FOUND: 0;
   return error;
 }
@@ -1723,6 +1749,7 @@ int ha_myisam::info(uint flag)
 	MI_KEYDEF *mi_keydef = &file->s->keyinfo[i];
 	sen_index *senna = mi_keydef->senna;
 	if (senna) {
+	  key->is_senna = true;
 	  key->senna_flags = mi_keydef->senna_flags;
 	  key->senna_initial_n_segments = mi_keydef->senna_initial_n_segments;
 	  key->senna_keys_size = mi_keydef->senna_keys_size;
@@ -1733,7 +1760,8 @@ int ha_myisam::info(uint flag)
 	  key->senna_inv_chunk_size = mi_keydef->senna_inv_chunk_size;
 
 	  key->senna_encoding = senna_enc_mysql(mi_keydef->senna_encoding);
-	}
+	} else
+	  key->senna_flags = false;
       }
     }
 #endif /* ENABLE_SENNA */
@@ -1833,7 +1861,12 @@ THR_LOCK_DATA **ha_myisam::store_lock(THD *thd,
 
 void ha_myisam::update_create_info(HA_CREATE_INFO *create_info)
 {
+#ifdef ENABLE_SENNA
+  ha_myisam::info(HA_STATUS_AUTO | HA_STATUS_CONST |
+		  HA_STATUS_VARIABLE | HA_STATUS_SENNA);
+#else
   ha_myisam::info(HA_STATUS_AUTO | HA_STATUS_CONST);
+#endif
   if (!(create_info->used_fields & HA_CREATE_USED_AUTO))
   {
     create_info->auto_increment_value=auto_increment_value;
@@ -2053,6 +2086,7 @@ my_bool ha_myisam::register_query_cache_table(THD *thd, char *table_name,
                                               *engine_callback,
                                               ulonglong *engine_data)
 {
+  DBUG_ENTER("ha_myisam::register_query_cache_table");
   /*
     No call back function is needed to determine if a cached statement
     is valid or not.
@@ -2064,39 +2098,48 @@ my_bool ha_myisam::register_query_cache_table(THD *thd, char *table_name,
   */
   *engine_data= 0;
 
-  /*
-    If a concurrent INSERT has happened just before the currently processed
-    SELECT statement, the total size of the table is unknown.
-
-    To determine if the table size is known, the current thread's snap shot of
-    the table size with the actual table size are compared.
-
-    If the table size is unknown the SELECT statement can't be cached.
-  */
-  ulonglong actual_data_file_length;
-  ulonglong current_data_file_length;
-
-  /*
-    POSIX visibility rules specify that "2. Whatever memory values a
-    thread can see when it unlocks a mutex <...> can also be seen by any
-    thread that later locks the same mutex". In this particular case,
-    concurrent insert thread had modified the data_file_length in
-    MYISAM_SHARE before it has unlocked (or even locked)
-    structure_guard_mutex. So, here we're guaranteed to see at least that
-    value after we've locked the same mutex. We can see a later value
-    (modified by some other thread) though, but it's ok, as we only want
-    to know if the variable was changed, the actual new value doesn't matter
-  */
-  actual_data_file_length= file->s->state.state.data_file_length;
-  current_data_file_length= file->save_state.data_file_length;
-
-  if (current_data_file_length != actual_data_file_length)
+  if (file->s->concurrent_insert)
   {
-    /* Don't cache current statement. */
-    return FALSE;
+    /*
+      If a concurrent INSERT has happened just before the currently
+      processed SELECT statement, the total size of the table is
+      unknown.
+
+      To determine if the table size is known, the current thread's snap
+      shot of the table size with the actual table size are compared.
+
+      If the table size is unknown the SELECT statement can't be cached.
+
+      When concurrent inserts are disabled at table open, mi_open()
+      does not assign a get_status() function. In this case the local
+      ("current") status is never updated. We would wrongly think that
+      we cannot cache the statement.
+    */
+    ulonglong actual_data_file_length;
+    ulonglong current_data_file_length;
+
+    /*
+      POSIX visibility rules specify that "2. Whatever memory values a
+      thread can see when it unlocks a mutex <...> can also be seen by any
+      thread that later locks the same mutex". In this particular case,
+      concurrent insert thread had modified the data_file_length in
+      MYISAM_SHARE before it has unlocked (or even locked)
+      structure_guard_mutex. So, here we're guaranteed to see at least that
+      value after we've locked the same mutex. We can see a later value
+      (modified by some other thread) though, but it's ok, as we only want
+      to know if the variable was changed, the actual new value doesn't matter
+    */
+    actual_data_file_length= file->s->state.state.data_file_length;
+    current_data_file_length= file->save_state.data_file_length;
+
+    if (current_data_file_length != actual_data_file_length)
+    {
+      /* Don't cache current statement. */
+      DBUG_RETURN(FALSE);
+    }
   }
 
   /* It is ok to try to cache current statement. */
-  return TRUE;
+  DBUG_RETURN(TRUE);
 }
 #endif
