@@ -144,7 +144,7 @@ static int send_file(THD *thd)
   if (errmsg)
   {
     sql_print_error("Failed in send_file() %s", errmsg);
-    DBUG_PRINT("error", (errmsg));
+    DBUG_PRINT("error", ("%s", errmsg));
   }
   DBUG_RETURN(error);
 }
@@ -202,7 +202,7 @@ void adjust_linfo_offsets(my_off_t purge_offset)
 
 bool log_in_use(const char* log_name)
 {
-  int log_name_len = strlen(log_name) + 1;
+  size_t log_name_len = strlen(log_name) + 1;
   THD *tmp;
   bool result = 0;
 
@@ -364,7 +364,6 @@ void mysql_binlog_send(THD* thd, char* log_ident, my_off_t pos,
     name=0;					// Find first log
 
   linfo.index_file_offset = 0;
-  thd->current_linfo = &linfo;
 
   if (mysql_bin_log.find_log_pos(&linfo, name, 1))
   {
@@ -373,6 +372,10 @@ void mysql_binlog_send(THD* thd, char* log_ident, my_off_t pos,
     goto err;
   }
 
+  pthread_mutex_lock(&LOCK_thread_count);
+  thd->current_linfo = &linfo;
+  pthread_mutex_unlock(&LOCK_thread_count);
+
   if ((file=open_binlog(&log, log_file_name, &errmsg)) < 0)
   {
     my_errno= ER_MASTER_FATAL_ERROR_READING_BINLOG;
@@ -1041,7 +1044,7 @@ void kill_zombie_dump_threads(uint32 slave_server_id)
     if (tmp->command == COM_BINLOG_DUMP &&
        tmp->server_id == slave_server_id)
     {
-      pthread_mutex_lock(&tmp->LOCK_delete);	// Lock from delete
+      pthread_mutex_lock(&tmp->LOCK_thd_data);	// Lock from delete
       break;
     }
   }
@@ -1054,7 +1057,7 @@ void kill_zombie_dump_threads(uint32 slave_server_id)
       again. We just to do kill the thread ourselves.
     */
     tmp->awake(THD::KILL_QUERY);
-    pthread_mutex_unlock(&tmp->LOCK_delete);
+    pthread_mutex_unlock(&tmp->LOCK_thd_data);
   }
 }
 
@@ -1226,9 +1229,6 @@ bool change_master(THD* thd, MASTER_INFO* mi)
       DBUG_RETURN(TRUE);
     }
   }
-  mi->rli.group_master_log_pos = mi->master_log_pos;
-  DBUG_PRINT("info", ("master_log_pos: %lu", (ulong) mi->master_log_pos));
-
   /*
     Coordinates in rli were spoilt by the 'if (need_relay_log_purge)' block,
     so restore them to good values. If we left them to ''/0, that would work;
@@ -1240,6 +1240,7 @@ bool change_master(THD* thd, MASTER_INFO* mi)
     That's why we always save good coords in rli.
   */
   mi->rli.group_master_log_pos= mi->master_log_pos;
+  DBUG_PRINT("info", ("master_log_pos: %lu", (ulong) mi->master_log_pos));
   strmake(mi->rli.group_master_log_name,mi->master_log_name,
 	  sizeof(mi->rli.group_master_log_name)-1);
 
@@ -1283,8 +1284,8 @@ int cmp_master_pos(const char* log_file_name1, ulonglong log_pos1,
 		   const char* log_file_name2, ulonglong log_pos2)
 {
   int res;
-  uint log_file_name1_len=  strlen(log_file_name1);
-  uint log_file_name2_len=  strlen(log_file_name2);
+  size_t log_file_name1_len=  strlen(log_file_name1);
+  size_t log_file_name2_len=  strlen(log_file_name2);
 
   //  We assume that both log names match up to '.'
   if (log_file_name1_len == log_file_name2_len)
@@ -1338,7 +1339,6 @@ bool mysql_show_binlog_events(THD* thd)
       name=0;					// Find first log
 
     linfo.index_file_offset = 0;
-    thd->current_linfo = &linfo;
 
     if (mysql_bin_log.find_log_pos(&linfo, name, 1))
     {
@@ -1346,9 +1346,18 @@ bool mysql_show_binlog_events(THD* thd)
       goto err;
     }
 
+    pthread_mutex_lock(&LOCK_thread_count);
+    thd->current_linfo = &linfo;
+    pthread_mutex_unlock(&LOCK_thread_count);
+
     if ((file=open_binlog(&log, linfo.log_file_name, &errmsg)) < 0)
       goto err;
 
+    /*
+      to account binlog event header size
+    */
+    thd->variables.max_allowed_packet += MAX_LOG_EVENT_HEADER;
+
     pthread_mutex_lock(log_lock);
 
     /*
@@ -1359,7 +1368,6 @@ bool mysql_show_binlog_events(THD* thd)
        This code will fail on a mixed relay log (one which has Format_desc then
        Rotate then Format_desc).
     */
-
     ev = Log_event::read_log_event(&log,(pthread_mutex_t*)0,description_event);
     if (ev)
     {
@@ -1550,37 +1558,52 @@ err:
   DBUG_RETURN(TRUE);
 }
 
-
+/**
+   Load data's io cache specific hook to be executed
+   before a chunk of data is being read into the cache's buffer
+   The fuction instantianates and writes into the binlog
+   replication events along LOAD DATA processing.
+   
+   @param file  pointer to io-cache
+   @return 0
+*/
 int log_loaded_block(IO_CACHE* file)
 {
+  DBUG_ENTER("log_loaded_block");
   LOAD_FILE_INFO *lf_info;
-  uint block_len ;
-
-  /* file->request_pos contains position where we started last read */
-  char* buffer = (char*) file->request_pos;
-  if (!(block_len = (char*) file->read_end - (char*) buffer))
-    return 0;
-  lf_info = (LOAD_FILE_INFO*) file->arg;
+  uint block_len;
+  /* buffer contains position where we started last read */
+  char* buffer= (char*) my_b_get_buffer_start(file);
+  uint max_event_size= current_thd->variables.max_allowed_packet;
+  lf_info= (LOAD_FILE_INFO*) file->arg;
   if (lf_info->last_pos_in_file != HA_POS_ERROR &&
-      lf_info->last_pos_in_file >= file->pos_in_file)
-    return 0;
-  lf_info->last_pos_in_file = file->pos_in_file;
-  if (lf_info->wrote_create_file)
+      lf_info->last_pos_in_file >= my_b_get_pos_in_file(file))
+    DBUG_RETURN(0);
+  
+  for (block_len= (uint) (my_b_get_bytes_in_buffer(file)); block_len > 0;
+       buffer += min(block_len, max_event_size),
+       block_len -= min(block_len, max_event_size))
   {
-    Append_block_log_event a(lf_info->thd, lf_info->thd->db, buffer,
-                             block_len, lf_info->log_delayed);
-    mysql_bin_log.write(&a);
+    lf_info->last_pos_in_file= my_b_get_pos_in_file(file);
+    if (lf_info->wrote_create_file)
+    {
+      Append_block_log_event a(lf_info->thd, lf_info->thd->db, buffer,
+                               min(block_len, max_event_size),
+                               lf_info->log_delayed);
+      mysql_bin_log.write(&a);
+    }
+    else
+    {
+      Begin_load_query_log_event b(lf_info->thd, lf_info->thd->db,
+                                   buffer,
+                                   min(block_len, max_event_size),
+                                   lf_info->log_delayed);
+      mysql_bin_log.write(&b);
+      lf_info->wrote_create_file= 1;
+      DBUG_SYNC_POINT("debug_lock.created_file_event",10);
+    }
   }
-  else
-  {
-    Begin_load_query_log_event b(lf_info->thd, lf_info->thd->db,
-                                 buffer, block_len,
-                                 lf_info->log_delayed);
-    mysql_bin_log.write(&b);
-    lf_info->wrote_create_file = 1;
-    DBUG_SYNC_POINT("debug_lock.created_file_event",10);
-  }
-  return 0;
+  DBUG_RETURN(0);
 }
 
 #endif /* HAVE_REPLICATION */
